package com.viettel.campaign.service.impl;

import com.viettel.campaign.model.Campaign;
import com.viettel.campaign.repository.CampaignRepository;
import com.viettel.campaign.repository.impl.CampaignRepositoryImpl;
import com.viettel.campaign.service.CampaignService;
import com.viettel.campaign.utils.Constants;
import com.viettel.campaign.web.dto.CampaignDTO;
import com.viettel.campaign.web.dto.ResultDTO;
import com.viettel.campaign.web.dto.request_dto.CampaignRequestDTO;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.modelmapper.ModelMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Transactional(rollbackFor = Exception.class)
public class CampaignServiceImpl implements CampaignService {
    private static final Logger logger = LogManager.getLogger(CampaignRepositoryImpl.class);

    @Autowired
    CampaignRepository campaignRepository;

    @Autowired
    ModelMapper modelMapper;

    @Override
    public Map searchCampaignExecute(int page, int pageSize, String sort, String agentId) {
        Map result = new HashMap();

        Pageable pageable = PageRequest.of(page, pageSize, Sort.by(sort));
        List<CampaignDTO> lst = campaignRepository.searchCampaignExecute(agentId, pageable);
        List<CampaignDTO> count = campaignRepository.searchCampaignExecute(agentId, null);

        result.put("totalItem", count.size());
        result.put("data", lst);

        return result;
    }

    @Override
    public ResultDTO search(CampaignRequestDTO requestDto) {
        return campaignRepository.search(requestDto);
    }

    @Override
    public ResultDTO findByCampaignCode(CampaignRequestDTO requestDTO) {
        return campaignRepository.findByCampaignCode(requestDTO);
    }

    @Override
    public ResultDTO addNewCampaign(CampaignDTO campaignDTO) {
        ResultDTO resultDTO = new ResultDTO();
        Campaign campaign = modelMapper.map(campaignDTO, Campaign.class);
        try {
            campaign = campaignRepository.save(campaign);
            //insert sub tables

            resultDTO.setErrorCode(Constants.ApiErrorCode.SUCCESS);
            resultDTO.setErrorCode(Constants.ApiErrorDesc.SUCCESS);
            resultDTO.setData(campaign);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDTO.setErrorCode(Constants.ApiErrorCode.ERROR);
            resultDTO.setErrorCode(Constants.ApiErrorDesc.ERROR);
        }
        return resultDTO;
    }
}
